/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* ===========
* = probe.c =
* ===========
*
* Description:
*   This application dumps the device-specific information that each
*   installed ATM driver registers with the CMM.
*
* Arguments:
*   none.
*
* Authors:
*   TLR - Theodore L. Ross
*
* Modification History:
*   Date          Name     Description
*   22-Sep-1996   TLR      Created.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <netaal/aal.h>

/* Return the textual interpretation of the PHY type value. */

char *phy_string (int p)
   {
   switch (p)
      {
      case ATM_PHY_TI:    return "T1";
      case ATM_PHY_E1:    return "E1";
      case ATM_PHY_E3:    return "E3";
      case ATM_PHY_DS3:   return "DS3";
      case ATM_PHY_STS1:  return "STS1";
      case ATM_PHY_100:   return "Generic 100 Mb/sec";
      case ATM_PHY_TAXI:  return "TAXI";
      case ATM_PHY_FC:    return "Fiber Channel";
      case ATM_PHY_STS3:  return "STS3c";
      case ATM_PHY_155:   return "Generic 155 Mb/sec";
      case ATM_PHY_STS12: return "STS12";
      default:            return "<unknown>";
      }
  }

/* Print the symbol for each bit in the capabilities bit mask. */

void print_cap (int p)
   {
   if (p & ATM_CAP_DOES_PTI)     printf ("PTI ");
   if (p & ATM_CAP_DOES_GFC)     printf ("GFC ");
   if (p & ATM_CAP_DOES_AAL5)    printf ("AAL5 ");
   if (p & ATM_CAP_DOES_AAL3)    printf ("AAL3/4 ");
   if (p & ATM_CAP_DOES_RAW)     printf ("RAW ");
   if (p & ATM_CAP_DOES_STAMP0)  printf ("STAMP0 ");
   if (p & ATM_CAP_DOES_CBR)     printf ("CBR ");
   if (p & ATM_CAP_DOES_VBR)     printf ("VBR ");
   if (p & ATM_CAP_DOES_PACING)  printf ("PACING");

   printf ("\n");
   }

main (int argc, char **argv)
   {
   int              sock, i;
   char             buff[100];
   unsigned int     device_count;
   ATM_DEVICE_INFO  info;

   /* Prevent garbage array data by clearing out the buff array. */

   for (i = 0; i < 100; i++)
      buff[i] = '/0';

   /* Open a native ATM socket. */

   sock = socket (AF_AAL, SOCK_DGRAM, 0);
   if (sock == -1)
      {
      /* If the error returned is EAFNOSUPPORT, the sockets API has not
       * been properly installed in the system.
       */
      if (errno == EAFNOSUPPORT)
         {
         printf ("ATM Sockets API not installed.\n");
         exit (1);
         }

      /* Some other error occurred on the socket call, print out errno. */

      printf ("[ socket call failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Get the number of physical ATM adapters enabled on the system. */

   if (ioctl (sock, SIO_GET_ATM_DEVICE_COUNT, &device_count) == -1)
      {
      printf ("[ SIO_GET_ATM_DEVICE_COUNT ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Do not continue if no ATM interfaces were discovered. */

   if (device_count == 0)
      {
      printf ("No ATM Interfaces Found.\n");
      exit (0);
      }

   /* Display the number of ATM interfaces discovered. */

   printf ("Found %d ATM Interfaces.\n", device_count);

   /* For each adapter (plus one more for testing), get the device info. */

   for (i = 0; i <= device_count; i++)
      {
      info.device_number   = i;
      info.driver_name     = buff;
      info.driver_name_len = 100;
      if (ioctl (sock, SIO_GET_DEVICE_INFO, &info) == -1)
         {
         /* The OS variable errno has value ENODEV when i == device_count.  
         *  Otherwise, there has been an error.
         */

         if ((errno != ENODEV) || (i != device_count))
            printf ("Device %d inaccessible, socket error: %d\n", i, errno);

         continue;
         }

      /* Dump the discovered NIC information to the display device. */

      buff[info.driver_name_len] = '\0';

      printf ("\n");
      printf ("  Device Number: %d\n", i);
      printf ("    Driver Name: '%s'\n", buff);
      printf ("       Phy Type: %s\n", phy_string (info.phy_type));
      printf ("         VC Max: %d\n", info.vc_max);
      printf ("  VCs Available: %d\n", info.vc_avail);
      printf ("        Max VCI: %d\n", info.vci_max);
      printf ("        Max VPI: %d\n", info.vpi_max);
      printf ("    Highest VCI: %d\n", info.vci_highest);
      printf ("    Highest VPI: %d\n", info.vpi_highest);
      printf ("   Capabilities: ");    print_cap (info.capabilities);
      printf ("       Hard MTU: %d\n", info.hard_mtu);
      printf ("        Max PCR: %d\n", info.peak_max);
      printf ("        Max SCR: %d\n", info.sust_max);
      printf ("      Max Burst: %d\n", info.burst_max);
      printf ("  Available PCR: %d\n", info.peak_avail);
      printf ("  Available SCR: %d\n", info.sust_avail);
      printf ("Available Burst: %d\n", info.burst_avail);
      }

   /* Close the socket resource and exit. */

   close (sock);
   }

